@@ -45,6 +45,43 @@
 #' m2 <- stepwise(m1)
 #' m2
 
+drop_func  <- function(criterion){
+    switch(criterion,
+           "aic" ={
+               list(
+                   opt.op    = which.min,
+                   comp.op   = `<`,
+                   out.str = "change.AIC",
+                   crit.str  = "aic")},
+           "test"={
+               list(
+                   opt.op    = which.max,
+                   comp.op   = `>`,
+                   out.str = "p.value",
+                   crit.str  = "p.value")
+           })
+}
+
+
+add_func  <- function(criterion){
+    switch(criterion,
+           "aic" ={
+               list(
+                   opt.op    = which.max,
+                   comp.op   = `>`,
+                   out.str = "change.AIC",
+                   crit.str  = "aic")},
+           "test"={
+               list(
+                   opt.op    = which.min,
+                   comp.op   = `<`,
+                   out.str = "p.value",
+                   crit.str  = "p.value")
+           })   
+}
+
+
+
 #' @export
 #' @rdname stepwise
 stepwise.iModel <- function(object,
@@ -113,23 +150,14 @@ backward <- function(object, criterion="aic", alpha=NULL, type="decomposable", s
 
   if (is.null(alpha))
     alpha <- if (criterion=="aic") 0 else 0.05
-  
-  switch(criterion,
-       "aic" ={opt.op    <- which.min
-               comp.op   <- `<`
-               outstring <- "change.AIC"
-               crit.str  <- "aic"},
-       "test"={opt.op    <- which.max
-               comp.op   <- `>`
-               outstring <- "p.value"
-               crit.str  <- "p.value"
-               })
 
+  ff <- drop_func(criterion)
+  
   t0 <- proc.time()
 
-  testEdges <- switch(search,
-                     "headlong" = {.testInEdges_headlong},
-                     "all"      = {.testInEdges_all}) 
+  ## testEdges <- switch(search,
+  ##                    "headlong" = {.testInEdges_headlong},
+  ##                    "all"      = {.testInEdges_all}) 
   itcount <- 1
   repeat{
       ##cat("Iteration", itcount, "\n")
@@ -144,22 +172,22 @@ backward <- function(object, criterion="aic", alpha=NULL, type="decomposable", s
           break
       } 
 
-      testMAT <- testEdges(object, edgeMAT, comp.op=comp.op, crit.str=crit.str,
-                           alpha=alpha, k=k, amat=amat, ...)
-      statvec   <- testMAT[, crit.str]
-      opt.idx   <- opt.op(statvec)
+      testMAT <- .test_in_edges(object, edgeMAT, comp.op=ff$comp.op, crit.str=ff$crit.str,
+                               alpha=alpha, k=k, search=search, amat=amat, ...)
+      statvec   <- testMAT[, ff$crit.str]
+      opt.idx   <- ff$opt.op(statvec)
 
       ## print(testMAT); print(opt.idx)
       
       if (details>=2) print(testMAT, row.names=FALSE, digits=4)
       ##str(list(statvec=statvec, opt.idx=opt.idx))
-      if (comp.op( statvec[opt.idx], alpha)) {
+      if (ff$comp.op( statvec[opt.idx], alpha)) {
           ## update object
           opt.edge <- as.character(testMAT[opt.idx, c("V1", "V2")])
           ## print(opt.edge)
           object   <- update(object, list(drop.edge=opt.edge))
           if (details >= 1) cat(sprintf("  %s %9.4f Edge deleted: %s\n",
-                                      outstring, statvec[opt.idx], .toString(opt.edge)))          
+                                      ff$out.str, statvec[opt.idx], .toString(opt.edge)))          
           if (itcount == steps) { break }
       } else break
       itcount <- itcount + 1
@@ -188,22 +216,13 @@ forward <- function(object, criterion="aic", alpha=NULL, type="decomposable", se
     if (is.null(alpha))
         alpha <- if (criterion=="aic") 0 else 0.05
     
-    switch(criterion,
-           "aic" ={opt.op    <- which.min
-               comp.op   <- `<`
-               outstring <- "change.AIC"
-               crit.str  <- "aic"},
-           "test"={opt.op    <- which.min
-               comp.op   <- `<`
-               outstring <- "p.value"
-               crit.str  <- "p.value"
-           })
-
+    ff <- add_func(criterion)
+    
     t0 <- proc.time()
     
-    testEdges <- switch(search,
-                      "headlong" ={.testOutEdges_headlong},
-                      "all"      ={.testOutEdges_all})
+    ## testEdges <- switch(search,
+    ##                   "headlong" ={.testOutEdges_headlong},
+    ##                   "all"      ={.testOutEdges_all})
     itcount <- 1
     repeat{
         ##cat("Iteration", itcount, "\n")
@@ -218,41 +237,162 @@ forward <- function(object, criterion="aic", alpha=NULL, type="decomposable", se
             break
         } 
         
-        testMAT   <- testEdges(object, edgeMAT, comp.op=comp.op, crit.str=crit.str,
-                               alpha=alpha, k=k, amat=amat, ...)
+        testMAT   <- .test_out_edges(object, edgeMAT, comp.op=ff$comp.op, crit.str=ff$crit.str,
+                               alpha=alpha, k=k, search=search, amat=amat, ...)
         
-        statvec   <- testMAT[,crit.str]
-        opt.idx   <- opt.op(statvec)
+        statvec   <- testMAT[,ff$crit.str]
+        opt.idx   <- ff$opt.op(statvec)
         
         if (details >= 2) print(testMAT, row.names=FALSE, digits=4)        
-        if (comp.op( statvec[opt.idx], alpha)) {
+        if (ff$comp.op( statvec[opt.idx], alpha)) {
             opt.edge <- as.character(testMAT[opt.idx, c("V1","V2")])            
             object   <- update(object, list(add.edge=opt.edge)) ## Update model            
             if (details >= 1) cat(sprintf("  %s %9.4f Edge added: %s\n",
-                                          outstring, statvec[opt.idx], .toString(opt.edge)))
+                                          ff$out.str, statvec[opt.idx], .toString(opt.edge)))
             if (itcount == steps) { break }
         } else break
         itcount <- itcount + 1
     }
     object
 }
+
+
+backfor <- function(object,
+                          criterion="aic",
+                          alpha=NULL,
+                          type="decomposable",
+                          search="all",
+                          steps = 1000,
+                          k = 2,
+                          direction="backward",
+                          fixin=NULL,
+                          fixout=NULL,
+                          details=1,
+                          trace=2, ...)
+{
+
+    direction  <- match.arg(direction, c("backward", "forward"))
+    criterion  <- match.arg(criterion, c("aic",        "test"))
+    search     <- match.arg(search,    c("headlong",   "all"))
+
+    if (is.null(alpha))
+        alpha <- if (criterion=="aic") 0 else 0.05
+    
+    do_back <- identical(direction, "backward") ## Logical
+    
+    if (isGSD_glist(.glist(object))[2]){  
+        type <- match.arg(type,      c("decomposable", "unrestricted"))
+    } else {
+        type <- "unrestricted"
+    }
+    
+    type   <- match.arg(type,   c("decomposable", "unrestricted"))
+    
+    ## discrete is needed because checking for decomposability is special for mixed models.
+    disc <- getmi(object, "disc.names") ## FIXME: Simpler than above
+    vn   <- getmi(object, "varNames")
+    
+    isgsd  <- isGSD_glist(.glist(object), discrete=disc)    
+    if (!all(isgsd))  type <- "unrestricted"
+
+    if (do_back){    ## Backward
+        fmat <- if (!is.null(fixin)) .as_amat(.do_handlefix(fixin), vn=vn)
+        ff  <- drop_func(criterion)
+        testEdges <- .test_in_edges
+    } else {         ## Forward
+        fmat <- if (!is.null(fixout)) .as_amat(.do_handlefix(fixout), vn=vn)
+        ff  <- add_func(criterion)
+        testEdges <- .test_out_edges     
+    }
+
+    itcount <- 1    
+    repeat{
+        ##cat("Iteration", itcount, "\n")
+        amat    <- .as_amat(.glist(object)) ## FIXME generate amat yet another time
+        
+        ingraph <- do_back
+        edgeMAT <- getEdges(amat, type=type, ingraph=ingraph, discrete=disc)
+        
+        if (!is.null(fmat))
+            edgeMAT <- .update_edgeMAT(edgeMAT, fmat, vn)
+        
+        if (nrow(edgeMAT) == 0){
+            if (details >= 1){
+                if (do_back) cat(sprintf("No edges can be removed\n"))
+                else cat(sprintf("No edges can be added\n"))
+            }
+            break
+        } 
+        
+        testMAT   <- testEdges(object, edgeMAT, comp.op=ff$comp.op, crit.str=ff$crit.str,
+                               alpha=alpha, k=k, search=search, amat=amat, ...)
+        statvec   <- testMAT[, ff$crit.str]
+        opt.idx   <- ff$opt.op(statvec)
+        
+        if (details >= 2) print(testMAT, row.names=FALSE, digits=4)
+        
+        if (ff$comp.op(statvec[opt.idx], alpha)) {
+            opt.edge <- as.character(testMAT[opt.idx, c("V1", "V2")])
+            if (do_back){  ## Backward
+                object   <- update(object, list(drop.edge=opt.edge))
+                if (details >= 1)
+                    cat(sprintf("  %s %9.4f Edge deleted: %s\n",
+                                ff$out.str, statvec[opt.idx], .toString(opt.edge)))                
+            } else {        ## Forward
+                object   <- update(object, list(add.edge=opt.edge))
+                if (details >= 1) cat(sprintf("  %s %9.4f Edge added: %s\n",
+                                              ff$out.str, statvec[opt.idx], .toString(opt.edge)))                                                                                     
+            }
+            
+            if (itcount == steps) { break }
+        } else break
+        itcount <- itcount + 1
+    }
+    object 
+}
+
+
+
+
+    ## if(details>=1){
+    ##     cat("STEPWISE: ",
+    ##         "\n criterion:", criterion, if (criterion=="aic") paste("( k =", round(k,2),")"),
+    ##         "\n direction:", direction,
+    ##         "\n type     :", type,
+    ##         "\n search   :", search,
+    ##         "\n steps    :", steps, "\n")
+    ## }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 ### dot-functions below here
 
-.amat_subtract <- function(amat1, amat2){
-    if (is.null(amat2)) return(amat1)
-    amat1 <- amat1 - amat2
-    amat1[amat1 < 0L] <- 0L
-    amat1
-}
-
-## Requires that first element (row) is smaller than second element (row)
-.hash_edge <- function(em){
-    z <- c(1L, 100000L)
-    if (is.matrix(em) && nrow(em) == 2) colSums(em * z)
-    else if (is.numeric(em) && length(em) == 2)sum(em * z)
-}
 
 
 .edge_matrix <- function(amat, duplicates=FALSE, names=FALSE, long=FALSE){
@@ -270,34 +410,6 @@ forward <- function(object, criterion="aic", alpha=NULL, type="decomposable", se
     if (long) t.default(m) else m
 }
 
-# obs: A named array
-.iloglin <- function(obs, ...){
-    vn <- names(dimnames(obs))
-    fit <- lapply(vn, function(v) {tabMarg(obs, v)})
-    n <- sum(obs)
-    s <- lapply(fit, function(f) sum(f * log(f)))
-    tt <- obs * log(obs)
-    
-    t0 <- sum(tt[!is.na(tt)])
-    t1 <- sum(unlist(s))
-    t2 <- length(fit) * n * log(n)
-    dev <- 2 * (t0 - n * log(n) - t1 + t2)
-    list(lrt=dev,
-         df=prod(dim(obs)) - length(dim(obs)) - 1)
-}
-
-
-# x : generator liste
-.is_gsd <- function(x, amat=NULL, discrete=NULL){
-    if (is.null(amat)) amat <- .glist2amat(x)
-    cliq <- maxCliqueMAT(amat)[[1]]
-    ## Is graphical ?
-    isg <- all(unlist(lapply(cliq, function(cq) isin(x, cq))))
-    if (!isg) c(isg = FALSE, issd = FALSE)
-    else
-        c(isg = TRUE,
-          issd = length(mcs_markedMAT(amat, discrete = discrete)) > 0)
-} 
 
 
 .do_handlefix <- function(x){
@@ -322,7 +434,16 @@ forward <- function(object, criterion="aic", alpha=NULL, type="decomposable", se
     else stop("Do not know what to do\n")
 }
 
+
+
 .update_edgeMAT <- function(edgeMAT, fmat, vn){
+    .amat_subtract <- function(amat1, amat2){
+        if (is.null(amat2)) return(amat1)
+        amat1 <- amat1 - amat2
+        amat1[amat1 < 0L] <- 0L
+        amat1
+    }
+
     am   <- .as_amat(matrix2list(edgeMAT), vn=vn)
     am   <- .amat_subtract(am, fmat)
     edgeMAT <- .edge_matrix(am, names=TRUE, long=TRUE)
@@ -333,10 +454,129 @@ forward <- function(object, criterion="aic", alpha=NULL, type="decomposable", se
 
 
 
+## # x : generator liste
+## .is_gsd <- function(x, amat=NULL, discrete=NULL){
+##     if (is.null(amat)) amat <- .glist2amat(x)
+##     cliq <- maxCliqueMAT(amat)[[1]]
+##     ## Is graphical ?
+##     isg <- all(unlist(lapply(cliq, function(cq) isin(x, cq))))
+##     if (!isg) c(isg = FALSE, issd = FALSE)
+##     else
+##         c(isg = TRUE,
+##           issd = length(mcs_markedMAT(amat, discrete = discrete)) > 0)
+## } 
 
 
 
+## ## Requires that first element (row) is smaller than second element (row)
+## .hash_edge <- function(em){
+##     z <- c(1L, 100000L)
+##     if (is.matrix(em) && nrow(em) == 2) colSums(em * z)
+##     else if (is.numeric(em) && length(em) == 2)sum(em * z)
+## }
 
+## # obs: A named array
+## .iloglin <- function(obs, ...){
+##     vn <- names(dimnames(obs))
+##     fit <- lapply(vn, function(v) {tabMarg(obs, v)})
+##     n <- sum(obs)
+##     s <- lapply(fit, function(f) sum(f * log(f)))
+##     tt <- obs * log(obs)
+    
+##     t0 <- sum(tt[!is.na(tt)])
+##     t1 <- sum(unlist(s))
+##     t2 <- length(fit) * n * log(n)
+##     dev <- 2 * (t0 - n * log(n) - t1 + t2)
+##     list(lrt=dev,
+##          df=prod(dim(obs)) - length(dim(obs)) - 1)
+## }
+
+
+## BACKWARD / DROP
+
+  ## switch(criterion,
+  ##      "aic" ={opt.op    <- which.min
+  ##              comp.op   <- `<`
+  ##              out.str <- "change.AIC"
+  ##              crit.str  <- "aic"},
+  ##      "test"={opt.op    <- which.max
+  ##              comp.op   <- `>`
+  ##              out.str <- "p.value"
+  ##              crit.str  <- "p.value"
+  ##              })
+
+## FORWARD / ADD
+
+    ## switch(criterion,
+    ##        "aic" ={
+    ##            ##opt.op    <- which.min ## Changed june 2020
+    ##            ##comp.op   <- `<`       ## Changed june 2020
+    ##            opt.op <- which.max
+    ##            comp.op <- `>`
+    ##            out.str <- "change.AIC"
+    ##            crit.str  <- "aic"},
+    ##        "test"={opt.op    <- which.min
+    ##            comp.op   <- `<`
+    ##            out.str <- "p.value"
+    ##            crit.str  <- "p.value"
+    ##        })
+
+
+
+
+
+
+
+        
+        ## switch(criterion,
+        ##        "aic" ={opt.op    <- which.min
+        ##            comp.op   <- `<`
+        ##            out.str <- "change.AIC"
+        ##            crit.str  <- "aic"},
+        ##        "test"={opt.op    <- which.max
+        ##            comp.op   <- `>`
+        ##            out.str <- "p.value"
+        ##            crit.str  <- "p.value"
+        ##        })
+        ## switch(criterion,
+        ##        "aic" ={
+        ##            ##opt.op    <- which.min ## Changed june 2020
+        ##            ##comp.op   <- `<`       ## Changed june 2020
+        ##            opt.op <- which.max
+        ##            comp.op <- `>`
+        ##            out.str <- "change.AIC"
+        ##            crit.str  <- "aic"},
+        ##        "test"={opt.op    <- which.min
+        ##            comp.op   <- `<`
+        ##            out.str <- "p.value"
+        ##            crit.str  <- "p.value"
+        ##        })
+
+  ## switch(criterion,
+  ##      "aic" ={opt.op    <- which.min
+  ##              comp.op   <- `<`
+  ##              out.str <- "change.AIC"
+  ##              crit.str  <- "aic"},
+  ##      "test"={opt.op    <- which.max
+  ##              comp.op   <- `>`
+  ##              out.str <- "p.value"
+  ##              crit.str  <- "p.value"
+  ##              })
+
+
+    ## switch(criterion,
+    ##        "aic" ={
+    ##            ##opt.op    <- which.min ## Changed june 2020
+    ##            ##comp.op   <- `<`       ## Changed june 2020
+    ##            opt.op <- which.max
+    ##            comp.op <- `>`
+    ##            out.str <- "change.AIC"
+    ##            crit.str  <- "aic"},
+    ##        "test"={opt.op    <- which.min
+    ##            comp.op   <- `<`
+    ##            out.str <- "p.value"
+    ##            crit.str  <- "p.value"
+    ##        })
 
 
 
